unit SPGP;

{!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
Please absolutely read Steve's function references before
using this unit, especially regarding the string formats!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}

{

Translated from 'spgp.bas' contained in 'spgp2550.zip'
created by Steve Heller (srheller@oz.net),
downloadable from http://www.oz.net/~srheller/spgp/
which also contains a link to the appropriate reference
(http://www.oz.net/~srheller/spgp/bin/spgphtml.zip)
for 'spgp.bas'. It provides a Delphi 2.0+ interface to
Steve Heller's 'spgp.dll' as provided with 'spgp2550.zip'
and later versions.

NB:	KeyIsOnRing returns a boolean value opposed to Steve's
	implementation. If you need to get extended error
	information through GetErrorString you should change
	the function result type to longint assuming that zero
	is indicating the existence of the checked key opposed
	to ord(true)=1.
	Two new constants have been added to analysis results:
	PGPAnalyze_Min and PGPAnalyze_Max. They don't create
	any new values but only simplify the error handling.

	Booleans:
	Though Steve always uses 'As Long' declarations for
	numeric values there seems to be a problem when
	typecasting them to Delphi's Longbools (at least with
	Delphi 4, not with Delphi 2). Because of that all
	Longbools were changed to simple Booleans.

	String handling:
	Before calling SPGP SetLength() sets the buffer size
	to the given buffer length, after returning it simply
	adjusts the string length to the actual output length.
	To avoid any unpredictable results or access violations
	the string results will be empty strings in case of an
	error returned by SPGP. Make sure you have 'Huge strings'
	enabled in Delphi ($H+) as compiler option to enable
	the use of PChars and proper typecasting of strings.
	This is required for working with common dll-libraries.

Steve's public key:

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: 2.6.2

mQCNAzIr628AAAEEALTFRPI8tVysHujfOUmWObD5aRVtO4vG/H4OjKRi0fuX6Zwa
5gnZuz1vAKjofY6WyZ1asMW2miwm1Krgz6px4h/R7NVgdujAW5zG7kGn5Wcvrjxo
zR+rrd3J9zknyzPC6g8fcpRfJf5LMd2AfLDG/r2I79khnttIDE/8MJkAFkZtAAUT
tB1TLlIuIEhlbGxlciA8c3JoZWxsZXJAb3oubmV0PrQoU3RldmVuIFIuIEhlbGxl
ciA8ZmVsaXhub25Ad29sZmVuZXQuY29tPrQkU3RldmUgSGVsbGVyIDxmZWxpeG5v
bkB3b2xmZW5ldC5jb20+iQCVAwUQMjJodU/8MJkAFkZtAQHk9gP/QaDjdIszEyxE
jdlk6XvMgZU3jPwsKKgug6Iq8ekGOCSEx8Lq0+ZwRQmMsisHUgZbi0fFlHK7Sul7
CraX9T0zqaLjtbrn2S7fwyGt1BMJgPod1esgKOWR1Rz8tqbkZwjgo7lYoA8dF8B1
zfKCg84jGehkgGEupBNfwy/faGu6fdE=
=Om+v
-----END PGP PUBLIC KEY BLOCK-----

}

{##############################################################}

{

translated by: Michael in der Wiesche (idw.doc@t-online.de)

my public key (extracted using this unit):

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: PGPsdk version 1.1.1 (C) 1997 Pretty Good Privacy, Inc.

mQCNAjPaZkoAAAEEALzl52/lPyN/ADW4P/dQ5eu6JTwx7u+y8pl+cLhY0la4R2LB
PPfUxIRetULrtl1qnUSJj2lbG/I8pjl0pVOzderO1wkJ0L6EmgsmRXXiWoQWwOBg
aZUjUNpACVJBDl3I+jsJgiC8mFfGpmb1gLveIgjdczRUNcpk9vFyV1YtC4dpAAUR
tCxNaWNoYWVsIGluIGRlciBXaWVzY2hlIDxpZHcuZG9jQHQtb25saW5lLmRlPokA
lQMFEDWGDlwd+1EwkYv11QEBeZkD/14i4Lw7Vsq3jKi+0wp/xYICI/AdeKtQ8Njx
cpYCZUu/ED9w8JTyHPUh+fcQuE3hGoy72GxozHEz+Jl91U5WNewLSfmJTWPqq6r8
ItphSFlFGdpYvQZQbEjs5PkW1Lz4W6G1eLW/Xz//lSsWghOeaft4ZMedfROvfpNB
njgk0Qi0iQCVAgUQNXt4Y/FyV1YtC4dpAQHlVQP7Bs0L4yTB+EajLl15t8Q04z0A
26Fk/6W/dSf/KGVSxmP9P1XBpoP3PXeg/1lV+OfuY4RjygkfrMdTtvseyBEMpOcY
3a7VLBz6ruyn+Yf3dAjQpGGKg23LckgUx8onuliIx0haiy88GVLSNBehXnq2ZFGr
tqIxGJudiDJDsSPMZbc=
=Ghy9
-----END PGP PUBLIC KEY BLOCK-----

}

interface

uses Windows, SysUtils;

const
  SPGPLib = 'spgp.dll';

// note that while some of these constants have the same name as
// variables found in the pgp *.h files, they do not all have the same
// values -- these are not meant to be translations of those files


  // PGPVersion						// added for pascal implementation
  PGP60 = $3000000;					// actually the SDK API version 1.5

  // key types						// added for pascal implementation
  PGPPublic = 0;
  PGPPrivate = 1;

  // public key algorithms
  PGPPublicKeyAlgorithm_Invalid = 0;			// this (0) may not be correct
  PGPPublicKeyAlgorithm_RSA = 1;
  PGPPublicKeyAlgorithm_RSAEncryptOnly = 2;
  PGPPublicKeyAlgorithm_RSASignOnly = 3;
  PGPPublicKeyAlgorithm_ElGamal = 4;			// A.K.A. DH
  PGPPublicKeyAlgorithm_DSA = 5;			// A.K.A. DSS

  // symmetric ciphers
  PGPCipherAlgorithm_IDEA = 1;
  PGPCipherAlgorithm_3DES = 2;
  PGPCipherAlgorithm_CAST5 = 3;

  // hashing algorithms
  PGPHashAlgorithm_Default = 0;
  PGPHashAlgorithm_MD5 = 1;
  PGPHashAlgorithm_SHA = 2;
  PGPHashAlgorithm_RIPEMD160 = 3;
  PGPHashAlgorithm_SHADouble = 4;			// not available in PGP 6

  // trust levels
  PGPKeyTrust_Undefined = 0;
  PGPKeyTrust_Unknown = 1;
  PGPKeyTrust_Never = 2;
  PGPKeyTrust_Marginal = 5;
  PGPKeyTrust_Complete = 6;
  PGPKeyTrust_Ultimate = 7;

  // validity levels
  PGPValidity_Unknown = 0;
  PGPValidity_Invalid = 1;
  PGPValidity_Marginal = 2;
  PGPValidity_Complete = 3;

  // analysis results
  PGPAnalyze_Min = 0;					// added for pascal implementation

  PGPAnalyze_Encrypted = 0;				// encrypted message
  PGPAnalyze_Signed = 1;				// signed message
  PGPAnalyze_DetachedSignature = 2;			// detached signature
  PGPAnalyze_Key = 3;					// key data
  PGPAnalyze_Unknown = 4;				// non-pgp message
  {for AnalyzeEx/AnalyzeFileEx only}
  PGPAnalyze_EncryptedConventional = 5;			// like it says
  PGPAnalyze_EncryptedNoKeys = 6;			// key-encrypted to keys not on local ring

  PGPAnalyze_Max = 6;					// added for pascal implementation

  // signature status
  SIGNED_GOOD = 0;
  SIGNED_NOT = 1;
  SIGNED_BAD = 2;
  SIGNED_NO_KEY = 3;

  // preferences, old style
  PGPPref_PublicKeyring = 0;
  PGPPref_PrivateKeyring = 1;
  PGPPref_RandomSeedFile = 2;
  PGPPref_DefaultKeyID = 3;

// -------------------------------------------------------------------------------------------------

  // preferences, jeune ecole
  spgpPref_PublicKeyring = 1;
  spgpPref_PrivateKeyring = 2;
  spgpPref_RandomSeedFile = 4;
  spgpPref_DefaultKeyID = 8;
  spgpPref_GroupsFile = 16;

  // for spgpPreferencesDialog, these select which page to open
  spgpPrefsPage_GeneralPrefs = 0;
  spgpPrefsPage_KeyringPrefs = 1;
  spgpPrefsPage_EmailPrefs = 2;
  spgpPrefsPage_HotkeyPrefs = 3;
  spgpPrefsPage_KeyserverPrefs = 4;
  spgpPrefsPage_CAPrefs = 5;
  spgpPrefsPage_AdvancedPrefs = 6;

// Key Properties Flags
  // string properties
  spgpKeyProp_KeyID                    = $0000001;	//    1
  spgpKeyProp_UserID                   = $0000002;	//    2
  spgpKeyProp_Fingerprint              = $0000004;	//    4
  spgpKeyProp_CreationTimeStr          = $0000008;	//    8
  spgpKeyProp_ExpirationTimeStr        = $0000010;	//   16
  // numeric properties
  spgpKeyProp_Keybits                  = $0000080;	//  128
  spgpKeyProp_KeyAlg                   = $0000100;	//  256
  spgpKeyProp_Trust                    = $0000200;	//  512
  spgpKeyProp_Validity                 = $0000400;	// 1024
  spgpKeyProp_CreationTime             = $0000800;	// 2048
  spgpKeyProp_ExpirationTime           = $0001000;	// 4096
  // boolean properties
  spgpKeyProp_IsSecret                 = $0008000;	// & cetera
  spgpKeyProp_IsAxiomatic              = $0010000;
  spgpKeyProp_IsRevoked                = $0020000;
  spgpKeyProp_IsDisabled               = $0040000;
  spgpKeyProp_IsExpired	               = $0080000;
  spgpKeyProp_IsSecretShared           = $0100000;	// PGP 6.x only
  spgpKeyProp_CanEncrypt               = $0200000;
  spgpKeyProp_CanDecrypt               = $0400000;	// PGP 6.x only
  spgpKeyProp_CanSign                  = $0800000;
  spgpKeyProp_CanVerify                = $1000000;	// PGP 6.x only
  spgpKeyProp_HasRevoker               = $2000000;	// DSA keys only
  spgpKeyProp_HasADK                   = $4000000;	// DSA keys only
  spgpKeyProp_HasSubKey                = $8000000;	// DSA keys only

  {MAX_PATH = 260; 					// this one is already defined in Windows.pas}

type
  TPathArray = Array[0..MAX_PATH-1] of Char;		// added for pascal implementation
  TKeyHexIDArray = Array[0..9] of Char;			// added for pascal implementation
  PPreferenceRec = ^TPreferenceRec;			// added for pascal implementation
  TPreferenceRec = Record
    PublicKeyring: TPathArray;
    PrivateKeyring: TPathArray;
    RandomSeedFile: TPathArray;
    GroupsFile: TPathArray;
    DefaultKeyHexID: TKeyHexIDArray;
  end;

// -------------------------------------------------------------------------------------------------


// encrypt
function Encode(const BufferIn: String; var BufferOut: String; BufferOutLen: Longint;
		Encrypt, Sign: Boolean; SignAlg: Longint;
		Conventional: Boolean; ConventionalAlg: Longint;
		Armor, TextMode, Clear: Boolean;
		const CryptKeyID, SignKeyID, SignKeyPass, ConventionalPass, Comment: String): Longint;
function EncodeFile(const FileIn, FileOut: String;
		    Encrypt, Sign: Boolean; SignAlg: Longint;
		    Conventional: Boolean; ConventionalAlg: Longint;
		    Armor, TextMode, Clear: Boolean;
		    const CryptKeyID, SignKeyID, SignKeyPass, ConventionalPass, Comment: String): Longint;
function _Encode(const BufferIn: String; var BufferOut: String; BufferOutLen: Longint;
		 Encrypt, Sign: Boolean; SignAlg: Longint;
		 Conventional: Boolean; ConventionalAlg: Longint;
		 Armor, TextMode, Clear, Compress, EyesOnly, MIME: Boolean;
		 const CryptKeyID, SignKeyID, SignKeyPass,
		 ConventionalPass, Comment: String;
                 var MIMESeparator: String): Longint;
function _EncodeFile(const FileIn, FileOut: String;
		     Encrypt, Sign: Boolean; SignAlg: Longint;
		     Conventional: Boolean; ConventionalAlg: Longint;
		     Armor, TextMode, Clear, Compress, EyesOnly, MIME: Boolean;
		     const CryptKeyID, SignKeyID, SignKeyPass,
		     ConventionalPass, Comment: String;
		     var MIMESeparator: String): Longint;

// decrypt
function Decode(const BufferIn: String; var BufferOut: String; BufferOutLen: Longint;
		const Pass: String; var SigProps: String): Longint;
function DecodeFile(const FileIn, FileOut, Pass: String; var SigProps: String): Longint;

// key export
function KeyExport(const KeyID: String; var BufferOut: String; BufferOutLen: Longint;
		   ExportPrivate, ExportCompatible: Boolean): Longint;
function KeyExportFile(const KeyID, FileOut: String;
		       ExportPrivate, ExportCompatible: Boolean): Longint;

// key import
function KeyImport(const BufferIn: String; var KeyProps: String; KeyPropsLen: Longint): Longint;
function KeyImportFile(const FileIn: String; var KeyProps: String; KeyPropsLen: Longint): Longint;
function _KeyImport(const BufferIn: String; var KeyProps: String; KeyPropsLen: Longint;
		    Import, AllProps: Boolean): Longint;
function _KeyImportFile(const FileIn: String; var KeyProps: String; KeyPropsLen: Longint;
			Import, AllProps: Boolean): Longint;

// key properties
function KeyProps(const KeyID: String; var KeyProps: String; KeyPropsLen: Longint): Longint;
function KeyRingID(var BufferOut: String; BufferOutLen: Longint): Longint;
function KeyRingToFile(const FileOut: String): Longint;
function KeyPropUserID(const KeyID: String; var BufferOut: String; BufferOutLen: Longint): Longint;
function KeyPropSig(const UserID: String; var BufferOut: String; BufferOutLen: Longint): Longint;
function KeyPropADK(const KeyHexID: String; var ADKeyProps: String;
		    ADKeyPropsLen: Longint; var ADKeyCount: Longint; Flags: Longint): Longint;
function KeyPropRevocationKey(const KeyHexID: String; var RevKeyProps: String;
			      RevKeyPropsLen: Longint; var RevKeyCount: Longint; Flags: Longint): Longint;

// error strings
function GetErrorString(theError: Longint; var BufferOut: String): Longint;

// analyze
function Analyze(const BufferIn: String): Longint;
function AnalyzeFile(const FileIn: String): Longint;
function AnalyzeEx(const BufferIn: String; var BufferOut: String; BufferOutLen: Longint): Longint;
function AnalyzeFileEx(const FileIn: String; var BufferOut: String; BufferOutLen: Longint): Longint;

// preferences
function SetPreference(Preference: Longint; const BufferIn: String): Longint;
function GetPreference(Preference: Longint; var BufferOut: String): Longint;
function SetPreferences(const Prefs: TPreferenceRec; Flags: Longint): Longint;
function GetPreferences(const Prefs: TPreferenceRec; Flags: Longint): Longint;

// miscellaneous
function KeyIsOnRing(const KeyID: String): Boolean;
function KeyRingCount: Longint;
function APIVersion: Longint;
function Version: Longint;

// signatures
function DetachedSigCreate(const FileIn, SigFile, SignKeyID, SignKeyPass, Comment: String;
			   SignAlg: Longint; Armor: Boolean): Longint;
function DetachedSigVerify(const SigFile, SignedFile: String; var SigProps: String): Longint;

// key manipulation
function KeyGenerate(const UserID, Passphrase: String; var NewKeyHexID: String;
		     KeyAlg, CipherAlg, Size, Expires: Longint;
		     FastGeneration, FailWithoutEntropy: Boolean;
		     WindowHandle: Longint): Longint;
function SubKeyGenerate(const MasterKeyHexID, MasterKeyPass: String; var NewSubKeyHexID: String;
			KeyAlg, Size, ExpiresIn: Longint;
			FastGeneration, FailWithoutEntropy: Boolean;
			WindowHandle: Longint): Longint;
function KeyRemove(const KeyID: String): Longint;
function KeyEnable(const KeyID: String): Longint;
function KeyDisable(const KeyID: String): Longint;
function EstimatePassphraseQuality(const Passphrase: String): Longint;
function KeyPassChange(const KeyID, OldPhrase, NewPhrase: String): Longint;
function KeySign(const KeyHexID, UserID, SignHexID, SignKeyPass: String;
		 Expires: Longint; Exportable: Boolean; Trust, Validity: Longint): Longint;
function KeySigRemove(const KeyHexID, UserID, SignHexID: String): Longint;

// user interface - PGP 6.x only
function UIEncode(const BufferIn: String; var BufferOut: String; BufferOutLen: Longint;
		  Encrypt, Sign: Boolean; SignAlg: Longint;
		  Conventional: Boolean; ConventionalAlg: Longint;
		  Clear, Compress, EyesOnly, MIME: Boolean;
		  const Comment: String; var MIMESeparator: String;
		  WindowHandle: Longint): Longint;
function UIEncodeFile(const FileIn, FileOut: String;
		      Encrypt, Sign: Boolean; SignAlg: Longint;
		      Conventional: Boolean; ConventionalAlg: Longint;
		      Armor, TextMode, Clear, Compress, EyesOnly, MIME: Boolean;
		      const Comment: String; var MIMESeparator: String;
		      WindowHandle: Longint): Longint;

function UIKeyPassphraseDialog(const KeyID: String; var Passphrase: String; WindowHandle: Longint): Longint;

function UIRecipientsDialog(var RecipientHexID: String; RecipientHexIDLen: Longint;
			    const Caption, Reserved1: String; DisplayMarginalValidity: Boolean;
			    Reserved2, Reserved3, Reserved4, Reserved5,
			    WindowHandle: Longint): Longint;

function UISigningPassphraseDialog(var SelectedKeyHexID, SelectedKeyPass: String;
				   const DefaultKeyID: String; FindMatchingKey: Boolean;
				   WindowHandle: Longint): Longint;

function UIConfirmationPassphraseDialog(var Passphrase: String; ShowPassphraseQuality: Boolean;
					MinimumPassphraseQuality, MinimumPassphraseLength,
					WindowHandle: Longint): Longint;

function UISelectKeysDialog(const KeyID: String; var KeyProps: String; const Prompt: String;
			    KeyPropsLen: Longint; ShowKeyRing: Boolean;
			    Flags, WindowHandle: Longint): Longint;

function PreferencesDialog(ShowPage: Longint; WindowHandle: Longint): Longint;


implementation

const
  EMPTY = '';

// import from spgp.dll; function names exported from the dll are case-sensitive.

// encrypt
function spgpEncode(BufferIn, BufferOut: PChar; BufferOutLen: ULONG;
		    Encrypt, Sign, SignAlg, Conventional, ConventionalAlg, Armor, TextMode, Clear: ULONG;
		    CryptKeyID, SignKeyID, SignKeyPass, ConventionalPass, Comment: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpencode';
function spgpEncodeFile(FileIn, FileOut: PChar;
			Encrypt, Sign, SignAlg, Conventional, ConventionalAlg, Armor, TextMode, Clear: ULONG;
			CryptKeyID, SignKeyID, SignKeyPass, ConventionalPass, Comment: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpencodefile';
function spgp_Encode(BufferIn, BufferOut: PChar; BufferOutLen: ULONG;
		     Encrypt, Sign, SignAlg, Conventional, ConventionalAlg,
		     Armor, TextMode, Clear, Compress, EyesOnly, MIME: ULONG;
		     CryptKeyID, SignKeyID, SignKeyPass,
		     ConventionalPass, Comment, MIMESeparator: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgp_encode';
function spgp_EncodeFile(FileIn, FileOut: PChar;
			 Encrypt, Sign, SignAlg, Conventional, ConventionalAlg,
                         Armor, TextMode, Clear, Compress, EyesOnly, MIME: ULONG;
			 CryptKeyID, SignKeyID, SignKeyPass,
			 ConventionalPass, Comment, MIMESeparator: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgp_encodefile';

// decrypt
function spgpDecode(BufferIn, BufferOut: PChar; BufferOutLen: ULONG; Pass, SigProps: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpdecode';
function spgpDecodeFile(FileIn, FileOut: PChar; Pass, SigProps: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpdecodefile';

// key export
function spgpKeyExport(KeyID, BufferOut: PChar; BufferOutLen: ULONG;
		       ExportPrivate, ExportCompatible: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeyexport';
function spgpKeyExportFile(KeyID, FileOut: PChar;
			   ExportPrivate, ExportCompatible: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeyexportfile';

// key import
function spgpKeyImport(BufferIn, KeyProps: PChar; KeyPropsLen: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeyimport';
function spgpKeyImportFile(FileIn, KeyProps: PChar; KeyPropsLen: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeyimportfile';
function spgp_KeyImport(BufferIn, KeyProps: PChar; KeyPropsLen, Import, AllProps: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgp_keyimport';
function spgp_KeyImportFile(FileIn, KeyProps: PChar; KeyPropsLen, Import, AllProps: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgp_keyimportfile';

// key properties
function spgpKeyProps(KeyID, KeyProps: PChar; KeyPropsLen: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeyprops';
function spgpKeyRingID(BufferOut: PChar; BufferOutLen: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeyringid';
function spgpKeyRingToFile(FileOut: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpkeyringtofile';
function spgpKeyPropUserID(KeyID, BufferOut: PChar; BufferOutLen: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeypropuserid';
function spgpKeyPropSig(UserID, BufferOut: PChar; BufferOutLen: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeypropsig';
function spgpKeyPropADK(KeyHexID, ADKeyProps: PChar; ADKeyPropsLen: ULONG;
			var ADKeyCount: Longint; Flags: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeypropadk';
function spgpKeyPropRevocationKey(KeyHexID, RevKeyProps: PChar; RevKeyPropsLen: ULONG;
				  var RevKeyCount: Longint; Flags: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeyproprevocationkey';

// error strings
function spgpGetErrorString(theError: ULONG; BufferOut: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpgeterrorstring';

// analyze
function spgpAnalyze(BufferIn: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpanalyze';
function spgpAnalyzeFile(FileIn: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpanalyzefile';
function spgpAnalyzeEx(BufferIn, BufferOut: PChar; BufferOutLen: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpanalyzeex';
function spgpAnalyzeFileEx(FileIn, BufferOut: PChar; BufferOutLen: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpanalyzefileex';

// preferences
function spgpSetPreference(Preference: ULONG; BufferIn: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpsetpreference';
function spgpGetPreference(Preference: ULONG; BufferOut: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpgetpreference';
function spgpSetPreferences(Prefs: PPreferenceRec; Flags: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpsetpreferences';
function spgpGetPreferences(Prefs: PPreferenceRec; Flags: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpgetpreferences';

// miscellaneous
function spgpKeyIsOnRing(KeyID: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpkeyisonring';
function spgpKeyRingCount: ULONG; stdcall;
	 external SPGPLib name 'spgpkeyringcount';
function spgpAPIVersion: ULONG; stdcall;
	 external SPGPLib name 'spgpsdkapiversion';
function spgpVersion: ULONG; stdcall;
	 external SPGPLib name 'spgpversion';

// signatures
function spgpDetachedSigCreate(FileIn, SigFile, SignKeyID, SignKeyPass, Comment: PChar;
			       SignAlg, Armor: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpdetachedsigcreate';
function spgpDetachedSigVerify(SigFile, SignedFile, SigProps: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpdetachedsigverify';

// key manipulation
function spgpKeyGenerate(UserID, Passphrase, NewKeyHexID: PChar;
			 KeyAlg, CipherAlg, Size, Expires,
			 FastGeneration, FailWithoutEntropy,
			 WindowHandle: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeygenerate';
function spgpSubKeyGenerate(MasterKeyHexID, MasterKeyPass, NewSubKeyHexID: PChar;
			    KeyAlg, Size, ExpiresIn,
			    FastGeneration, FailWithoutEntropy,
			    WindowHandle: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpsubkeygenerate';
function spgpKeyRemove(KeyID: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpkeyremove';
function spgpKeyEnable(KeyID: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpkeyenable';
function spgpKeyDisable(KeyID: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpkeydisable';
function spgpEstimatePassphraseQuality(Passphrase: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpestimatepassphrasequality';
function spgpKeyPassChange(KeyID, OldPhrase, NewPhrase: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpkeypasschange';
function spgpKeySign(KeyHexID, UserID, SignHexID, SignKeyPass: PChar;
		     Expires, Exportable, Trust, Validity: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpkeysign';
function spgpKeySigRemove(KeyHexID, UserID, SignHexID: PChar): ULONG; stdcall;
	 external SPGPLib name 'spgpkeysigremove';

// user interface - PGP 6.x only
function spgpUIEncode(BufferIn, BufferOut: PChar; BufferOutLen: ULONG;
		      Encrypt, Sign, SignAlg, Conventional, ConventionalAlg,
		      Clear, Compress, EyesOnly, MIME: ULONG;
		      Comment, MIMESeparator: PChar; WindowHandle: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpuiencode';
function spgpUIEncodeFile(FileIn, FileOut: PChar;
			  Encrypt, Sign, SignAlg, Conventional, ConventionalAlg,
			  Armor, Texmode, Clear, Compress, EyesOnly, MIME: ULONG;
			  Comment, MIMESeparator: PChar; WindowHandle: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpuiencodefile';

function spgpUIKeyPassphraseDialog(KeyID, Passphrase: PChar; WindowHandle: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpuikeypassphrasedialog';

function spgpUIRecipientsDialog(RecipientHexID: PChar; RecipientHexIDLen: ULONG;
				Caption, Reserved1: PChar; DisplayMarginalValidity,
				Reserved2, Reserved3, Reserved4, Reserved5, WindowHandle: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpuirecipientsdialog';

function spgpUISigningPassphraseDialog(SelectedKeyHexID, SelectedKeyPass, DefaultKeyID: PChar;
				       FindMatchingKey, WindowHandle: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpuisigningpassphrasedialog';

function spgpUIConfirmationPassphraseDialog(Passphrase: PChar; ShowPassphraseQuality,
					    MinimumPassphraseQuality, MinimumPassphraseLength,
					    WindowHandle: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpuiconfirmationpassphrasedialog';

function spgpUISelectKeysDialog(KeyID, KeyProps, Prompt: PChar;
				KeyPropsLen, ShowKeyRing, Flags, WindowHandle: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgpuiselectkeysdialog';

function spgpPreferencesDialog(ShowPage, WindowHandle: ULONG): ULONG; stdcall;
	 external SPGPLib name 'spgppreferencesdialog';


// pascal implementation (Delphi 2.0+)

// encrypt

function Encode(const BufferIn: String; var BufferOut: String; BufferOutLen: Longint;
		Encrypt, Sign: Boolean; SignAlg: Longint;
		Conventional: Boolean; ConventionalAlg: Longint;
		Armor, TextMode, Clear: Boolean;
		const CryptKeyID, SignKeyID, SignKeyPass, ConventionalPass, Comment: String): Longint;
begin
  SetLength(BufferOut, BufferOutLen);
  Result:=spgpEncode(PChar(BufferIn), PChar(BufferOut), BufferOutLen,
		     ord(Encrypt), ord(Sign), SignAlg,
		     ord(Conventional), ConventionalAlg,
		     ord(Armor), ord(TextMode), ord(Clear),
		     PChar(CryptKeyID), PChar(SignKeyID), PChar(SignKeyPass),
		     PChar(ConventionalPass), PChar(Comment));
  if Result=0 then
    SetLength(BufferOut, StrLen(PChar(BufferOut)))
  else BufferOut:=EMPTY;
end;

function EncodeFile(const FileIn, FileOut: String;
		    Encrypt, Sign: Boolean; SignAlg: Longint;
		    Conventional: Boolean; ConventionalAlg: Longint;
		    Armor, TextMode, Clear: Boolean;
		    const CryptKeyID, SignKeyID, SignKeyPass, ConventionalPass, Comment: String): Longint;
begin
  Result:=spgpEncodeFile(PChar(FileIn), PChar(FileOut),
			 ord(Encrypt), ord(Sign), SignAlg,
			 ord(Conventional), ConventionalAlg,
			 ord(Armor), ord(TextMode), ord(Clear),
			 PChar(CryptKeyID), PChar(SignKeyID), PChar(SignKeyPass),
			 PChar(ConventionalPass), PChar(Comment));
end;

function _Encode(const BufferIn: String; var BufferOut: String; BufferOutLen: Longint;
		 Encrypt, Sign: Boolean; SignAlg: Longint;
		 Conventional: Boolean; ConventionalAlg: Longint;
		 Armor, TextMode, Clear, Compress, EyesOnly, MIME: Boolean;
		 const CryptKeyID, SignKeyID, SignKeyPass,
		 ConventionalPass, Comment: String;
                 var MIMESeparator: String): Longint;
begin
  SetLength(BufferOut, BufferOutLen);
  SetLength(MIMESeparator, 81);
  Result:=spgp_Encode(PChar(BufferIn), PChar(BufferOut), BufferOutLen,
	              ord(Encrypt), ord(Sign), SignAlg,
		      ord(Conventional), ConventionalAlg,
		      ord(Armor), ord(TextMode), ord(Clear),
		      ord(Compress), ord(EyesOnly), ord(MIME),
		      PChar(CryptKeyID), PChar(SignKeyID), PChar(SignKeyPass),
		      PChar(ConventionalPass), PChar(Comment), PChar(MIMESeparator));
  if Result=0 then
    SetLength(BufferOut, StrLen(PChar(BufferOut)))
  else BufferOut:=EMPTY;
  if MIME and (Result=0) then
    SetLength(MIMESeparator, StrLen(PChar(MIMESeparator)))
  else MIMESeparator:=EMPTY;
end;

function _EncodeFile(const FileIn, FileOut: String;
		     Encrypt, Sign: Boolean; SignAlg: Longint;
		     Conventional: Boolean; ConventionalAlg: Longint;
		     Armor, TextMode, Clear, Compress, EyesOnly, MIME: Boolean;
		     const CryptKeyID, SignKeyID, SignKeyPass,
		     ConventionalPass, Comment: String;
                     var MIMESeparator: String): Longint;
begin
  SetLength(MIMESeparator, 81);
  Result:=spgp_EncodeFile(PChar(FileIn), PChar(FileOut),
			  ord(Encrypt), ord(Sign), SignAlg,
			  ord(Conventional), ConventionalAlg,
			  ord(Armor), ord(TextMode), ord(Clear),
			  ord(Compress), ord(EyesOnly), ord(MIME),
			  PChar(CryptKeyID), PChar(SignKeyID), PChar(SignKeyPass),
			  PChar(ConventionalPass), PChar(Comment), PChar(MIMESeparator));
  if MIME and (Result=0) then
    SetLength(MIMESeparator, StrLen(PChar(MIMESeparator)))
  else MIMESeparator:=EMPTY;
end;

// decrypt

function Decode(const BufferIn: String; var BufferOut: String; BufferOutLen: Longint;
		const Pass: String; var SigProps: String): Longint;
begin
  SetLength(SigProps, 256);
  SetLength(BufferOut, BufferOutLen);
  Result:=spgpDecode(PChar(BufferIn), PChar(BufferOut), BufferOutLen, PChar(Pass), PChar(SigProps));
  if Result=0 then begin
    SetLength(BufferOut, StrLen(PChar(BufferOut)));
    SetLength(SigProps, StrLen(PChar(SigProps)));
  end
  else begin
    BufferOut:=EMPTY;
    SigProps:=EMPTY;
  end;
end;

function DecodeFile(const FileIn, FileOut, Pass: String; var SigProps: String): Longint;
begin
  SetLength(SigProps, 256);
  Result:=spgpDecodeFile(PChar(FileIn), PChar(FileOut), PChar(Pass), PChar(SigProps));
  if Result=0 then
    SetLength(SigProps, StrLen(PChar(SigProps)))
  else SigProps:=EMPTY;
end;

// key export

function KeyExport(const KeyID: String; var BufferOut: String; BufferOutLen: Longint;
		   ExportPrivate, ExportCompatible: Boolean): Longint;
begin
  SetLength(BufferOut, BufferOutLen);
  Result:=spgpKeyExport(PChar(KeyID), PChar(BufferOut), BufferOutLen,
  			ord(ExportPrivate), ord(ExportCompatible));
  if Result=0 then
    SetLength(BufferOut, StrLen(PChar(BufferOut)))
  else BufferOut:=EMPTY;
end;

function KeyExportFile(const KeyID, FileOut: String;
		       ExportPrivate, ExportCompatible: Boolean): Longint;
begin
  Result:=spgpKeyExportFile(PChar(KeyID), PChar(FileOut),
  			    ord(ExportPrivate), ord(ExportCompatible));
end;

// key import

function KeyImport(const BufferIn: String; var KeyProps: String; KeyPropsLen: Longint): Longint;
begin
  SetLength(KeyProps, KeyPropsLen);
  Result:=spgpKeyImport(PChar(BufferIn), PChar(KeyProps), KeyPropsLen);
  if Result=0 then
    SetLength(KeyProps, StrLen(PChar(KeyProps)))
  else KeyProps:=EMPTY;
end;

function KeyImportFile(const FileIn: String; var KeyProps: String; KeyPropsLen: Longint): Longint;
begin
  SetLength(KeyProps, KeyPropsLen);
  Result:=spgpKeyImportFile(PChar(FileIn), PChar(KeyProps), KeyPropsLen);
  if Result=0 then
    SetLength(KeyProps, StrLen(PChar(KeyProps)))
  else KeyProps:=EMPTY;
end;

function _KeyImport(const BufferIn: String; var KeyProps: String; KeyPropsLen: Longint;
		    Import, AllProps: Boolean): Longint;
begin
  SetLength(KeyProps, KeyPropsLen);
  Result:=spgp_KeyImport(PChar(BufferIn), PChar(KeyProps), KeyPropsLen, ord(Import), ord(AllProps));
  if Result=0 then
    SetLength(KeyProps, StrLen(PChar(KeyProps)))
  else KeyProps:=EMPTY;
end;

function _KeyImportFile(const FileIn: String; var KeyProps: String; KeyPropsLen: Longint;
			Import, AllProps: Boolean): Longint;
begin
  SetLength(KeyProps, KeyPropsLen);
  Result:=spgp_KeyImportFile(PChar(FileIn), PChar(KeyProps), KeyPropsLen, ord(Import), ord(AllProps));
  if Result=0 then
    SetLength(KeyProps, StrLen(PChar(KeyProps)))
  else KeyProps:=EMPTY;
end;

// key properties

function KeyProps(const KeyID: String; var KeyProps: String; KeyPropsLen: Longint): Longint;
begin
  SetLength(KeyProps, KeyPropsLen);
  Result:=spgpKeyProps(PChar(KeyID), PChar(KeyProps), KeyPropsLen);
  if Result=0 then
    SetLength(KeyProps, StrLen(PChar(KeyProps)))
  else KeyProps:=EMPTY;
end;

function KeyRingID(var BufferOut: String; BufferOutLen: Longint): Longint;
begin
  SetLength(BufferOut, BufferOutLen);
  Result:=spgpKeyRingID(PChar(BufferOut), BufferOutLen);
  if Result=0 then
    SetLength(BufferOut, StrLen(PChar(BufferOut)))
  else BufferOut:=EMPTY;
end;

function KeyRingToFile(const FileOut: String): Longint;
begin
  Result:=spgpKeyRingToFile(PChar(FileOut));
end;

function KeyPropUserID(const KeyID: String; var BufferOut: String; BufferOutLen: Longint): Longint;
begin
  SetLength(BufferOut, BufferOutLen);
  Result:=spgpKeyPropUserID(PChar(KeyID), PChar(BufferOut), BufferOutLen);
  if Result=0 then
    SetLength(BufferOut, StrLen(PChar(BufferOut)))
  else BufferOut:=EMPTY;
end;

function KeyPropSig(const UserID: String; var BufferOut: String; BufferOutLen: Longint): Longint;
begin
  SetLength(BufferOut, BufferOutLen);
  Result:=spgpKeyPropSig(PChar(UserID), PChar(BufferOut), BufferOutLen);
  if Result=0 then
    SetLength(BufferOut, StrLen(PChar(BufferOut)))
  else BufferOut:=EMPTY;
end;

function KeyPropADK(const KeyHexID: String; var ADKeyProps: String;
		    ADKeyPropsLen: Longint; var ADKeyCount: Longint; Flags: Longint): Longint;
begin
  SetLength(ADKeyProps, ADKeyPropsLen);
  Result:=spgpKeyPropADK(PChar(KeyHexID), PChar(ADKeyProps), ADKeyPropsLen, ADKeyCount, Flags);
  if Result=0 then
    SetLength(ADKeyProps, StrLen(PChar(ADKeyProps)))
  else ADKeyProps:=EMPTY;
end;

function KeyPropRevocationKey(const KeyHexID: String; var RevKeyProps: String;
			      RevKeyPropsLen: Longint; var RevKeyCount: Longint; Flags: Longint): Longint;
begin
  SetLength(RevKeyProps, RevKeyPropsLen);
  Result:=spgpKeyPropRevocationKey(PChar(KeyHexID), PChar(RevKeyProps), RevKeyPropsLen, RevKeyCount, Flags);
  if Result=0 then
    SetLength(RevKeyProps, StrLen(PChar(RevKeyProps)))
  else RevKeyProps:=EMPTY;
end;

// error strings

function GetErrorString(theError: Longint; var BufferOut: String): Longint;
begin
  SetLength(BufferOut, 256);
  Result:=spgpGetErrorString(theError, PChar(BufferOut));
  if (Result=0) or (Result=-11997) then
    SetLength(BufferOut, StrLen(PChar(BufferOut)))
  else BufferOut:=EMPTY;
end;

// analyze

function Analyze(const BufferIn: String): Longint;
begin
  Result:=spgpAnalyze(PChar(BufferIn));
end;

function AnalyzeFile(const FileIn: String): Longint;
begin
  Result:=spgpAnalyzeFile(PChar(FileIn));
end;

function AnalyzeEx(const BufferIn: String; var BufferOut: String; BufferOutLen: Longint): Longint;
begin
  SetLength(BufferOut, BufferOutLen);
  Result:=spgpAnalyzeEx(PChar(BufferIn), PChar(BufferOut), BufferOutLen);
  if (Result>=PGPAnalyze_Min) and (Result<=PGPAnalyze_Max) then
    SetLength(BufferOut, StrLen(PChar(BufferOut)))
  else BufferOut:=EMPTY;
end;

function AnalyzeFileEx(const FileIn: String; var BufferOut: String; BufferOutLen: Longint): Longint;
begin
  SetLength(BufferOut, BufferOutLen);
  Result:=spgpAnalyzeFileEx(PChar(FileIn), PChar(BufferOut), BufferOutLen);
  if (Result>=PGPAnalyze_Min) and (Result<=PGPAnalyze_Max) then
    SetLength(BufferOut, StrLen(PChar(BufferOut)))
  else BufferOut:=EMPTY;
end;

// preferences

function SetPreference(Preference: Longint; const BufferIn: String): Longint;
begin
  Result:=spgpSetPreference(Preference, PChar(BufferIn));
end;

function GetPreference(Preference: Longint; var BufferOut: String): Longint;
begin
  SetLength(BufferOut, 256);
  Result:=spgpGetPreference(Preference, PChar(BufferOut));
  if Result=0 then
    SetLength(BufferOut, StrLen(PChar(BufferOut)))
  else BufferOut:=EMPTY;
end;

function SetPreferences(const Prefs: TPreferenceRec; Flags: Longint): Longint;
begin
  Result:=spgpSetPreferences(@Prefs, Flags);
end;

function GetPreferences(const Prefs: TPreferenceRec; Flags: Longint): Longint;
begin
  Result:=spgpGetPreferences(@Prefs, Flags);
end;


// miscellaneous

function KeyIsOnRing(const KeyID: String): Boolean;
begin
  Result:=(spgpKeyIsOnRing(PChar(KeyID))=0);
end;

function KeyRingCount: Longint;
begin
  Result:=spgpKeyRingCount;
end;

function APIVersion: Longint;
begin
  Result:=spgpAPIVersion;
end;

function Version: Longint;
begin
  Result:=spgpVersion;
end;

// signatures

function DetachedSigCreate(const FileIn, SigFile, SignKeyID, SignKeyPass, Comment: String;
			   SignAlg: Longint; Armor: Boolean): Longint;
begin
  Result:=spgpDetachedSigCreate(PChar(FileIn), PChar(SigFile), PChar(SignKeyID), PChar(SignKeyPass),
  				PChar(Comment), SignAlg, ord(Armor));
end;

function DetachedSigVerify(const SigFile, SignedFile: String; var SigProps: String): Longint;
begin
  SetLength(SigProps, 256);
  Result:=spgpDetachedSigVerify(PChar(SigFile), PChar(SignedFile), PChar(SigProps));
  if Result=0 then
    SetLength(SigProps, StrLen(PChar(SigProps)))
  else SigProps:=EMPTY;
end;

// key manipulation

function KeyGenerate(const UserID, Passphrase: String; var NewKeyHexID: String;
		     KeyAlg, CipherAlg, Size, Expires: Longint;
		     FastGeneration, FailWithoutEntropy: Boolean;
		     WindowHandle: Longint): Longint;
begin
  SetLength(NewKeyHexID, 256);
  Result:=spgpKeyGenerate(PChar(UserID), PChar(Passphrase), PChar(NewKeyHexID),
			  KeyAlg, CipherAlg, Size, Expires,
			  ord(FastGeneration), ord(FailWithoutEntropy),
			  WindowHandle);
  if Result=0 then
    SetLength(NewKeyHexID, StrLen(PChar(NewKeyHexID)))
  else NewKeyHexID:=EMPTY;
end;

function SubKeyGenerate(const MasterKeyHexID, MasterKeyPass: String; var NewSubKeyHexID: String;
			KeyAlg, Size, ExpiresIn: Longint;
			FastGeneration, FailWithoutEntropy: Boolean;
			WindowHandle: Longint): Longint;
begin
  SetLength(NewSubKeyHexID, 256);
  Result:=spgpSubKeyGenerate(PChar(MasterKeyHexID), PChar(MasterKeyPass), PChar(NewSubKeyHexID),
			     KeyAlg, Size, ExpiresIn,
			     ord(FastGeneration), ord(FailWithoutEntropy),
			     WindowHandle);
  if Result=0 then
    SetLength(NewSubKeyHexID, StrLen(PChar(NewSubKeyHexID)))
  else NewSubKeyHexID:=EMPTY;
end;

function KeyRemove(const KeyID: String): Longint;
begin
  Result:=spgpKeyRemove(PChar(KeyID));
end;

function KeyEnable(const KeyID: String): Longint;
begin
  Result:=spgpKeyEnable(PChar(KeyID));
end;

function KeyDisable(const KeyID: String): Longint;
begin
  Result:=spgpKeyDisable(PChar(KeyID));
end;

function EstimatePassphraseQuality(const Passphrase: String): Longint;
begin
  Result:=spgpEstimatePassphraseQuality(PChar(Passphrase));
end;

function KeyPassChange(const KeyID, OldPhrase, NewPhrase: String): Longint;
begin
  Result:=spgpKeyPassChange(PChar(KeyID), PChar(OldPhrase), PChar(NewPhrase));
end;

function KeySign(const KeyHexID, UserID, SignHexID, SignKeyPass: String;
		 Expires: Longint; Exportable: Boolean; Trust, Validity: Longint): Longint;
begin
  Result:=spgpKeySign(PChar(KeyHexID), PChar(UserID), PChar(SignHexID), PChar(SignKeyPass),
		      Expires, ord(Exportable), Trust, Validity);
end;

function KeySigRemove(const KeyHexID, UserID, SignHexID: String): Longint;
begin
  Result:=spgpKeySigRemove(PChar(KeyHexID), PChar(UserID), PChar(SignHexID));
end;

// user interface - PGP 6.x only

function UIEncode(const BufferIn: String; var BufferOut: String; BufferOutLen: Longint;
		  Encrypt, Sign: Boolean; SignAlg: Longint;
		  Conventional: Boolean; ConventionalAlg: Longint;
		  Clear, Compress, EyesOnly, MIME: Boolean;
		  const Comment: String; var MIMESeparator: String; WindowHandle: Longint): Longint;
begin
  SetLength(BufferOut, BufferOutLen);
  SetLength(MIMESeparator, 81);
  Result:=spgpUIEncode(PChar(BufferIn), PChar(BufferOut), BufferOutLen,
	               ord(Encrypt), ord(Sign), SignAlg,
		       ord(Conventional), ConventionalAlg,
		       ord(Clear), ord(Compress), ord(EyesOnly), ord(MIME),
		       PChar(Comment), PChar(MIMESeparator), WindowHandle);
  if Result=0 then
    SetLength(BufferOut, StrLen(PChar(BufferOut)))
  else BufferOut:=EMPTY;
  if MIME and (Result=0) then
    SetLength(MIMESeparator, StrLen(PChar(MIMESeparator)))
  else MIMESeparator:=EMPTY;
end;

function UIEncodeFile(const FileIn, FileOut: String;
		      Encrypt, Sign: Boolean; SignAlg: Longint;
		      Conventional: Boolean; ConventionalAlg: Longint;
		      Armor, Textmode, Clear, Compress, EyesOnly, MIME: Boolean;
		      const Comment: String; var MIMESeparator: String; WindowHandle: Longint): Longint;
begin
  SetLength(MIMESeparator, 81);
  Result:=spgpUIEncodeFile(PChar(FileIn), PChar(FileOut),
			   ord(Encrypt), ord(Sign), SignAlg,
			   ord(Conventional), ConventionalAlg,
			   ord(Armor), ord(Textmode), ord(Clear), ord(Compress), ord(EyesOnly), ord(MIME),
			   PChar(Comment), PChar(MIMESeparator), WindowHandle);
  if MIME and (Result=0) then
    SetLength(MIMESeparator, StrLen(PChar(MIMESeparator)))
  else MIMESeparator:=EMPTY;
end;

function UIKeyPassphraseDialog(const KeyID: String; var Passphrase: String; WindowHandle: Longint): Longint;
begin
  SetLength(Passphrase, 256);
  Result:=spgpUIKeyPassphraseDialog(PChar(KeyID), PChar(Passphrase), WindowHandle);
  if Result=0 then
    SetLength(Passphrase, StrLen(PChar(Passphrase)))
  else Passphrase:=EMPTY;
end;

function UIRecipientsDialog(var RecipientHexID: String; RecipientHexIDLen: Longint;
			    const Caption, Reserved1: String; DisplayMarginalValidity: Boolean;
			    Reserved2, Reserved3, Reserved4, Reserved5, WindowHandle: Longint): Longint;
begin
  SetLength(RecipientHexID, RecipientHexIDLen);
  Result:=spgpUIRecipientsDialog(PChar(RecipientHexID), RecipientHexIDLen, PChar(Caption), nil,
				 ord(DisplayMarginalValidity), 0, 0, 0, 0, WindowHandle);
  if Result=0 then
    SetLength(RecipientHexID, StrLen(PChar(RecipientHexID)))
  else RecipientHexID:=EMPTY;
end;

function UISigningPassphraseDialog(var SelectedKeyHexID, SelectedKeyPass: String; const DefaultKeyID: String;
				   FindMatchingKey: Boolean; WindowHandle: Longint): Longint;
begin
  SetLength(SelectedKeyHexID, 256);
  SetLength(SelectedKeyPass, 256);
  Result:=spgpUISigningPassphraseDialog(PChar(SelectedKeyHexID), PChar(SelectedKeyPass), PChar(DefaultKeyID),
  					ord(FindMatchingKey), WindowHandle);
  if Result=0 then begin
    SetLength(SelectedKeyHexID, StrLen(PChar(SelectedKeyHexID)));
    SetLength(SelectedKeyPass, StrLen(PChar(SelectedKeyPass)));
  end
  else begin
    SelectedKeyHexID:=EMPTY;
    SelectedKeyPass:=EMPTY;
  end;
end;

function UIConfirmationPassphraseDialog(var Passphrase: String; ShowPassphraseQuality: Boolean;
					MinimumPassphraseQuality, MinimumPassphraseLength,
					WindowHandle: Longint): Longint;
begin
  SetLength(Passphrase, 256);
  Result:=spgpUIConfirmationPassphraseDialog(PChar(Passphrase), ord(ShowPassphraseQuality),
					     MinimumPassphraseQuality, MinimumPassphraseLength, WindowHandle);
  if Result=0 then
    SetLength(Passphrase, StrLen(PChar(Passphrase)))
  else Passphrase:=EMPTY;
end;

function UISelectKeysDialog(const KeyID: String; var KeyProps: String; const Prompt: String;
			    KeyPropsLen: Longint; ShowKeyRing: Boolean;
			    Flags, WindowHandle: Longint): Longint;
begin
  SetLength(KeyProps, KeyPropsLen);
  Result:=spgpUISelectKeysDialog(PChar(KeyID), PChar(KeyProps), PChar(Prompt),
				 KeyPropsLen, ord(ShowKeyRing), Flags, WindowHandle);
  if Result=0 then
    SetLength(KeyProps, StrLen(PChar(KeyProps)))
  else KeyProps:=EMPTY;
end;

function PreferencesDialog(ShowPage: Longint; WindowHandle: Longint): Longint;
begin
  Result:=spgpPreferencesDialog(ShowPage, WindowHandle);
end;

end.

