{$A+,B-,C-,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$MINSTACKSIZE $00004000}
{$MAXSTACKSIZE $00100000}
{$IMAGEBASE $00400000}
{$APPTYPE GUI}
unit SampleUnit;

{

This sample provides code for implementing the most important key functions
with SPGP through my import unit for SPGP 2.5.5.0. It's been written carefully
to avoid any problems that might compromise running sytems, but it doesn't
contain a sufficient error handling for every possible situation as it only
should serve for demonstration purposes. So be careful when using parts of
it in your own applications. You'll find the main parts in 'PGPFuncs'.

Michael in der Wiesche, Oct. 04th, 2000

}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, StdCtrls, ExtCtrls, ComCtrls, Dialogs;

type
  TSample = class(TForm)
    Panel: TPanel;
    ComboBox: TComboBox;
    RadioGroup: TRadioGroup;
    RichEdit: TRichEdit;
    OpenDialog: TOpenDialog;
    Button: TButton;
    GroupBox: TGroupBox;
    ListBox: TListBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ComboBoxChange(Sender: TObject);
    procedure RadioGroupClick(Sender: TObject);
    procedure ButtonClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    slUserNames: TStringList;
    slUserIDs: TStringList;
    sKeyFile: String;
  public
  end;

var
  Sample: TSample;

implementation

uses PGPFuncs, SPGP;

{$R *.DFM}

procedure TSample.FormCreate(Sender: TObject);
begin
  slUserNames:=TStringList.Create;
  slUserIDs:=TStringList.Create;
  GetKeyLists(E, slUserNames, slUserIDs);
  ComboBox.Items.Assign(slUserNames);
end;

procedure TSample.FormDestroy(Sender: TObject);
begin
  slUserNames.Free;
  slUserIDs.Free;
end;

procedure TSample.ComboBoxChange(Sender: TObject);
var iErrorCode: Integer; sUser, sResult: String;
begin
  iErrorCode:=0;
  if ComboBox.ItemIndex<>-1 then begin
    sUser:=slUserIDs.Strings[ComboBox.ItemIndex];
    Application.ProcessMessages;
    case RadioGroup.ItemIndex of
      0: begin
	RichEdit.Alignment:=taLeftJustify;
	iErrorCode:=ExtractKey(sUser, sResult);
	if iErrorCode=0 then RichEdit.Text:=NL1 + sUser + NL2 + sResult;
      end;
      1: begin
	RichEdit.Alignment:=taCenter;
	iErrorCode:=GetFingerprint(sUser, sResult);
	if iErrorCode=0 then RichEdit.Text:=NL1 + sResult;
      end;
    end;
  end;
  if iErrorCode<>0 then begin
    GetErrorString(iErrorCode, sResult);
    RichEdit.Text:=sResult;
  end;
  RichEdit.Refresh;
end;

procedure TSample.RadioGroupClick(Sender: TObject);
begin
  ComboBoxChange(Sender);
end;

procedure TSample.ButtonClick(Sender: TObject);
var iErrorCode: Integer; sResult: String;
begin
  with OpenDialog do begin
    InitialDir:=ExtractFilePath(Application.ExeName);
    if Execute then begin
      Application.ProcessMessages;
      iErrorCode:=GetKeyLists(FileName, slUserNames, slUserIDs);
      if iErrorCode<>0 then
        GetErrorString(iErrorCode, sResult)
      else with ListBox do begin
        Items.Assign(slUserNames);
        GroupBox.BringToFront;
        sKeyFile:=FileName;
        SetFocus;
      end;
      RichEdit.Text:=NL1 + sResult;
    end;
  end;
end;

procedure TSample.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var iErrorCode: Integer; sResult: String;
begin
  case Key of
    VK_RETURN: with ListBox do begin
      Screen.Cursor:=crHourGlass;
      iErrorCode:=AddSelKeys(sKeyFile, ListBox, slUserNames, slUserIDs, sResult);
      if iErrorCode<>0 then
        GetErrorString(iErrorCode, sResult)
      else sResult:='Added key(s):' + NL2 + sResult;
      GetKeyLists(E, slUserNames, slUserIDs);
      ComboBox.Items.Assign(slUserNames);
      RichEdit.Text:=NL1 + sResult;
      Screen.Cursor:=crDefault;
      GroupBox.SendToBack;
      Application.ProcessMessages;
    end;
    VK_ESCAPE: GroupBox.SendToBack;
  end;
end;



end.
